/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  04 - Processing Queued Event Notifications.sql
*
*   Summary: Demonstrates how to process queued events from the Service
*            Broker queue for the Event Notification
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/


USE [EventNotificationsDB]
GO

-- Check the Queue
SELECT *
FROM EventNotificationQueue
ORDER BY queuing_order;
GO

-- Receive the next available message FROM the queue
DECLARE @message_body xml;

RECEIVE TOP(1) -- just handle one message at a time
 @message_body=message_body
 FROM EventNotificationQueue;

SELECT @message_body;
GO

-- Check the Queue
SELECT *
FROM EventNotificationQueue
ORDER BY queuing_order;
GO

/********** STOP **********/

-- Generate Multiple Events
RAISERROR (N'Test ERRORLOG Event Notifications', 10, 1) WITH LOG;
GO
RAISERROR (N'Test ERRORLOG Event Notifications', 10, 1) WITH LOG;
GO


-- Declare the table variable to hold the XML messages
DECLARE @messages TABLE
( message_data xml );

-- Receive the next available message FROM the queue
RECEIVE cast(message_body as xml)
FROM EventNotificationQueue
INTO @messages;

-- Parse the XML from the table variable
SELECT 
	message_data.value('(/EVENT_INSTANCE/EventType)[1]', 'varchar(128)' ) as EventType,
	message_data.value('(/EVENT_INSTANCE/PostTime)[1]', 'varchar(128)') AS PostTime,
	message_data.value('(/EVENT_INSTANCE/TextData)[1]', 'varchar(128)' ) AS TextData,
	message_data.value('(/EVENT_INSTANCE/Severity)[1]', 'varchar(128)' ) AS Severity,
	message_data.value('(/EVENT_INSTANCE/Error)[1]', 'varchar(128)' ) AS Error
FROM @messages;

SELECT * FROM @messages;
GO

-- Check the Queue
SELECT *
FROM EventNotificationQueue
ORDER BY queuing_order;
GO