/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  12 - DBM - Agent Job Automation.sql
*
*   Summary: Demonstrates using Event Notifications to automate enabling and
*            disabling SQL Agent jobs when a database mirroring failover
*            occurs changing the role of the servers.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

USE [EventNotificationsDB]
GO

-- Create an Activation Procedure for the Queue
CREATE PROCEDURE [dbo].[ProcessDBMStateChangeEventNotifications]
WITH EXECUTE AS OWNER
AS 
SET NOCOUNT ON
DECLARE @message_body xml 
DECLARE @email_message nvarchar(MAX)
WHILE (1 = 1)
BEGIN
	-- Receive the next available message FROM the queue
	WAITFOR (
		RECEIVE TOP(1) -- just handle one message at a time
			@message_body=message_body
			FROM dbo.[MirroringStateChangeQueue]
	), TIMEOUT 1000  -- if the queue is empty for one second, give UPDATE and go away
	-- If we didn't get anything, bail out
	IF (@@ROWCOUNT = 0)
		BEGIN
			BREAK
		END 

	DECLARE @PreviousState INT, @NewState INT, @DatabaseID INT
	
	SELECT @PreviousState = @message_body.value('(/EVENT_INSTANCE/IntegerData)[1]', 'int' ),
			@NewState = @message_body.value('(/EVENT_INSTANCE/State)[1]', 'int' ),
			@DatabaseID = @message_body.value('(/EVENT_INSTANCE/DatabaseID)[1]', 'int' )
			
	--SELECT 
	--CASE @PreviousState
	--	WHEN 0 THEN 'Null Notification'
	--	WHEN 1 THEN 'Synchronized Principal with Witness'
	--	WHEN 2 THEN 'Synchronized Principal without Witness'
	--	WHEN 3 THEN 'Synchronized Mirror with Witness'
	--	WHEN 4 THEN 'Synchronized Mirror without Witness'
	--	WHEN 5 THEN 'Connection with Principal Lost'
	--	WHEN 6 THEN 'Connection with Mirror Lost'
	--	WHEN 7 THEN 'Manual Failover'
	--	WHEN 8 THEN 'Automatic Failover'
	--	WHEN 9 THEN 'Mirroring Suspended'
	--	WHEN 10 THEN 'No Quorum'
	--	WHEN 11 THEN 'Synchronizing Mirror'
	--	WHEN 12 THEN 'Principal Running Exposed'
	--	WHEN 13 THEN 'Synchronizing Principal'
	--	ELSE 'Unknown'
	--END AS PreviousState
	
	IF (@PreviousState = 0 AND @NewState = 13) -- Database Starting Up as Principal Role
	BEGIN
		-- Enable the Backup Databases Job
		EXEC msdb.dbo.sp_update_job 
					@job_name = 'Backup Databases', 
					@enabled=1;
	END
	
	IF (@PreviousState = 1 AND (@NewState = 7 OR @NewState = 8))  -- Failover has occurred
	BEGIN
		-- Disable the Backup Databases Job
		EXEC msdb.dbo.sp_update_job 
					@job_name = 'Backup Databases', 
					@enabled=0;
	END

--	-- Generate formatted email message
--	SELECT @email_message = 'The following event was logged in the SQL Server ErrorLog:' + CHAR(10) +
--	'PostTime: ' + @message_body.value('(/EVENT_INSTANCE/PostTime)[1]', 'varchar(128)') + CHAR(10) +
--	'Error: ' + @message_body.value('(/EVENT_INSTANCE/Error)[1]', 'varchar(20)' )  + CHAR(10) +
--	'Severity: ' + @message_body.value('(/EVENT_INSTANCE/Severity)[1]', 'varchar(20)' ) + CHAR(10) +
--	'TextData: ' + @message_body.value('(/EVENT_INSTANCE/TextData)[1]', 'varchar(4000)' );
--	-- Send email using Database Mail
--    EXEC msdb.dbo.sp_send_dbmail
--             @profile_name = 'Gmail Notification Account', -- your defined email profile 
--             @recipients = 'jmkehayias@gmail.com', -- your email
--             @subject = 'SQL Server Error Log Event',
--             @body = @email_message;
--
--	END

END
GO

USE [EventNotificationsDB];
GO

-- Sign the procedure with the certificate's private key
ADD SIGNATURE TO OBJECT::[ProcessDBMStateChangeEventNotifications]
BY CERTIFICATE [DBMailCertificate] 
WITH PASSWORD = '$tr0ngp@$$w0rd';
GO



USE [EventNotificationsDB];
GO
--  Create a service broker queue to hold the events
CREATE QUEUE [MirroringStateChangeQueue]
	WITH STATUS=ON, 
		 ACTIVATION 
			(PROCEDURE_NAME = [ProcessDBMStateChangeEventNotifications],
			 MAX_QUEUE_READERS = 1,
			 EXECUTE AS OWNER) 
GO

--  Create a service broker service receive the events
CREATE SERVICE [MirroringStateChangeService]
    ON QUEUE [MirroringStateChangeQueue] ([http://schemas.microsoft.com/SQL/Notifications/PostEventNotification])
GO

CREATE ROUTE [MirroringStateChangeRoute]
    WITH SERVICE_NAME = 'MirroringStateChangeService',
    ADDRESS = 'LOCAL';
GO


CREATE EVENT NOTIFICATION [CaptureMirroringStateChanges]
    ON SERVER
    WITH FAN_IN
    FOR DATABASE_MIRRORING_STATE_CHANGE
    TO SERVICE 'MirroringStateChangeService', 'current database';
GO


/*
--  Cleanup objects
DROP EVENT NOTIFICATION CaptureMirroringStateChanges ON SERVER
DROP ROUTE MirroringStateChangeRoute
DROP SERVICE MirroringStateChangeService
DROP QUEUE MirroringStateChangeQueue 
*/
